#!/usr/bin/env node

/**
 * 
 * @fileOverview bash命令定义
 * @author leo.yy
 * 
 */

require('shelljs/global');
var co = require('co');
var fs = require('fs');
var path = require('path');
var request = require('request');
var colors = require('cli-color');
var inquirer = require('inquirer');
var program = require('commander');
var Promise = require('promise');
var urllib = require('urllib');

var thunkify = require('thunkify');
var handleApi = thunkify(request.post);
var execThunk = thunkify(exec);

var appInfo = require('./../package.json');
var webServer = require('./../lib/server');

var utils = require('../lib/utils');
var gitTools = utils.git;
var USERCONFIG = utils.getUserConfig;
var BUILDINFOS = utils.getBuildInfo(USERCONFIG.version);
var CHECKNPM = utils.npm.checkNpm

var LOGO = colors.xterm(20).bgXterm(226);

var Rx = require('rx');

var getAPIHost = function(env) {
	if (env == 'dev') {
		return `http://co.shuqudata.com:3004`
	} else if (env === 'daily') {
		return `http://toolsdaily.shuqudata.com`
	} else if (env === 'pre') {
		return `http://toolspro.shuqudata.com`
	} else if (env === 'production') {
		return `http://toolspro.shuqudata.com`
	} else {
		return false;
	}
};
// var dependencies = ' webpack gulp gulp-less gulp-uglify del gulp-jshint gulp-inline-source gulp-htmlmin gulp-inline-css gulp-replace jshint underscore gulp-util cli-color br-bid react react-dom redux react-redux redux-thunk';

var setConfigVersion = thunkify(function(version, callback) {
	var branch = version ? version : false;
	gitTools.setConfigVersionThunk(branch, function(err, config) {
		return callback(err, config);
	});
});

program
	.allowUnknownOption() //不报错误
	.version(appInfo.version)
	.usage('FEDTools前端开发工具')
	.option('-q, --quiet', '安静模式')
	.option('-r, --react', '初始化react工程')
	.action(function() {
		console.log(LOGO(' _____ _____ ____ _____           _     '));
		console.log(LOGO('|  ___| ____|  _ \\_   _|__   ___ | |___ '));
		console.log(LOGO('| |_  |  _| | | | || |/ _ \\ / _ \\| / __|'));
		console.log(LOGO('|  _| | |___| |_| || | (_) | (_) | \\__ \\'));
		console.log(LOGO('|_|   |_____|____/ |_|\\___/ \\___/|_|___/'));
	})
	.parse(process.argv);

program
	.command('lint')
	.alias('l')
	.description('代码检测')
	// .option('-i, --input [type]', '路径')
	.action(function(cmd, options) {
		console.log(colors.green('开始代码检测'));
		var deployJSON = null;
		co(function*() {
			inquirer.prompt([{
				type: 'checkbox',
				name: 'selectedEntry',
				message: '请选择需要进行代码检测的页面:',
				choices: BUILDINFOS.autoGetHtml.keys
			}]).then(function(answers) {
				deployJSON = answers;
				deployJSON.jsEntry = {};
				answers.selectedEntry.forEach(function(se, index) { // 生成发布列表，构建列表
					for (var htmlKey in BUILDINFOS.autoGetHtml.jsEntry) {
						if (htmlKey.split(se).length > 1) {
							if (BUILDINFOS.autoGetHtml.jsEntry[htmlKey]) {
								deployJSON.jsEntry[htmlKey] = BUILDINFOS.autoGetHtml.jsEntry[htmlKey];
							}
							break;
						}
					}
				});
				return deployJSON;
			}).then(function(data) {
				if (data.selectedEntry.length == 0) {
					return console.log(colors.red('没有选择任何页面,检测结束'));
				}
				co(function*() {
					var filename = path.join(utils.path.cwdPath, 'build.json');
					var jsonData = JSON.stringify(data);
					try {
						fs.writeFileSync(filename, jsonData);
						console.log(colors.green('build.json创建成功'));
					} catch (err) {
						console.log(colors.red('build.json写入失败，请检查该文件'));
						console.log(colors.red(JSON.stringify(err)));
					}
					var initTime = new Date().getTime();
					try {
						yield execThunk('gulp lint');
					} catch (err) {
						console.log(colors.red('代码检测异常！'));
						console.log(colors.red(err));
					}
					var nowTime = new Date().getTime();
					console.log(colors.blue('耗时:' + (nowTime - initTime) / 1000, 's'));
					console.log(colors.green('代码检测完成!'));
				});
			});
		});

	}).on('--help', function() {
		console.log('  举个栗子:');
		console.log('');
		console.log('    bid lint   :   js代码规范检测');
		console.log('');
		process.exit(1);
	});

program
	.command('version')
	.alias('v')
	.description('版本信息')
	// .option('-i, --input [type]', '路径')
	.action(function(cmd, options) {
		// co(function*() {
			console.log(colors.blue(`正在检测远端版本...`));
			console.log(colors.green(`当前版本为：${appInfo.version}`));
			CHECKNPM('br-bid', function(err, version, moduleInfo) {
				if (err) {
					console.log(colors.red(`检测失败...`));
					console.error(err);
					return;
				}
				console.log(colors.green(`最新版本为：${moduleInfo['dist-tags'].latest}`));
				if (moduleInfo['dist-tags'].latest != appInfo.version) {
					console.log(colors.yellow(`当前版本不是最新，请手动更新...`));
				} else {
					console.log(colors.green(`恭喜您，当前版本是最新版本.`));
				}
			});
		// });

	}).on('--help', function() {
		console.log('  举个栗子:');
		console.log('');
		console.log('    bid lint   :   js代码规范检测');
		console.log('');
		process.exit(1);
	});

program
	.command('build')
	.alias('p')
	.description('进行构建')
	.action(function(cmd, options) {
		var deployJSON = null; // build.json源
		var isPublish = false; // 是否构建后调用发布接口
		var configure = {}; // 读取自本地工程config.json配置

		co(function*() {
			try {
				configure = yield setConfigVersion(false); // 检测git分支，设置config.version，并返回新的分支
			} catch (e) {
				console.log(colors.red('警告：当前不是git开发环境！'))
			}
			inquirer.prompt([{
				type: 'input',
				name: 'username',
				message: '请输入您的用户名:'
			}, {
				type: 'list',
				name: 'env',
				message: '请选择发布环境:',
				choices: [{
						name: '日常发布（相对路径构建，同时构建发布js、html）',
						value: 'daily'
					}, {
						name: '预发发布（相对路径构建，同时构建发布js、html）',
						value: 'pre'
					}, {
						name: '线上发布（CDN路径构建，仅构建发布html）',
						value: 'production'
					}] //['本地生成部署配置', '日常环境', '预发环境', '线上环境']
			}, {
				type: 'checkbox',
				name: 'selectedEntry',
				message: '请选择需要进行构建的页面:',
				choices: BUILDINFOS.autoGetHtml.keys
			}]).then(function(answers) {
				deployJSON = answers;
				deployJSON.htmlEntry = [];
				deployJSON.jsEntry = {};
				deployJSON.appName = configure.appName; // 应用名
				deployJSON.remotes = configure.remotes; // Git远端地址
				deployJSON.version = configure.version; // Git分支版本
				deployJSON.publish = configure.publish; // 发布配置信息
				deployJSON.cdnhost = configure.cdnhost; // 静态资源cdn域名

				answers.selectedEntry.forEach(function(se, index) { // 生成发布列表，构建列表
					for (var htmlKey in BUILDINFOS.autoGetHtml.jsEntry) {
						if (htmlKey.split(se).length > 1) {
							var tmpSrc = './' + htmlKey + '.html';
							if (configure.version) {
								tmpSrc = tmpSrc.replace(configure.version + '/', '');
							}
							if (BUILDINFOS.autoGetHtml.jsEntry[htmlKey]) {
								deployJSON.jsEntry[htmlKey] = BUILDINFOS.autoGetHtml.jsEntry[htmlKey];
							}
							deployJSON.htmlEntry.push(tmpSrc);
							break;
						}
					}
				});
				return deployJSON;
			}).then(function(data) {
				if (data.selectedEntry.length == 0) {
					return console.log(colors.red('没有选择任何页面,构建结束'));
				}
				co(function*() {
					var filename = path.join(utils.path.cwdPath, 'build.json');
					console.log(colors.green('gulp deploy --entry ' + filename + ' --env ' + data.env));
					var jsonData = JSON.stringify(data);
					try {
						fs.writeFileSync(filename, jsonData);
						console.log(colors.green('build.json创建成功'));
					} catch (err) {
						console.log(colors.red('build.json写入失败，请检查该文件'));
						console.log(colors.red(JSON.stringify(err)));
					}

					try {
						yield execThunk('gulp deploy --entry ' + filename + ' --env daily'); // 在本地进行build
					} catch (e) {
						console.log(colors.red('本地线上构建失败！'));
						console.log(e);
					}

					var chmod777 = function(callback) {
						var start777 = new Date().getTime();
						exec('chmod -R 777 ./deploy', {
							async: true,
							silent: program.quiet
						}, function(code, output) {
							var end777 = new Date().getTime();
							console.log(colors.green('修改build权限777完成，共耗时:' + (end777 - start777) / 1000, 's'));
							// chmod777();
						});
					}

					chmod777();
					callbackFn = function(userName) {
						if (userName) { // 校验用户名
							var serverType = '默认';
							var doPublish = function(confArr) {
								var scpStartTime = new Date().getTime();
								// exec('scp -r ./build root@101.200.132.102:/home', {
								// exec('scp -r ./build/ ' + userName + '@192.168.180.10:/opt/www/minions', {
								// 由于服务器端免密钥或交互式shell需要运维配合，开发成本较高，必所以暂时使用手工创建日常服务器项目目录的办法；
								// 日常发布时，须保证服务器上已经存在项目文件夹，否则需要手动新建，并将owner设置为www,权限777,否则可能会影响日常发布
								// console.log('scp -r ./build/* ' + userName + '@' + publishHost + ':' + publishPath + USERCONFIG.appName)
								var $path = confArr.path;
								confArr.host.forEach(function(host) {
									var scpCmd = 'scp -r ./deploy/build/* ' + userName + '@' + host + ':' + $path + USERCONFIG.appName
									console.log(scpCmd);
									exec(scpCmd, {
										async: true
									}, function(code, output) {
										var nowTime = new Date().getTime();
										console.log(colors.green('已成功上传到 [' + serverType + ']（' + host + '） 服务器!'));
										console.log(colors.blue('上传耗时:' + (nowTime - scpStartTime) / 1000, 's'));
									});
								});
							}
							try {
								if (data.env === 'daily') { // 发布日常
									serverType = '日常';
									doPublish(USERCONFIG.publish.daily)
								} else if (data.env === 'pre') { // 发布预发阿里云
									serverType = '预发';
									doPublish(USERCONFIG.publish.pre)
								} else if (data.env === 'production') { // 发布线上阿里云
									serverType = '线上';
									doPublish(USERCONFIG.publish.production);
								} else {
									colors.yellow('发布未成功，因为您没有指定正确的发布环境');
								}
							} catch (e) {
								console.log(colors.red('config.json发布配置错误，' + serverType + '发布失败'));
								console.log(colors.red(e));
							}
						} else {
							console.log(colors.red('上传失败，无法解析您输入的userName'));
						}
					}
					callbackFn(data.username);
					console.log(colors.green('构建完毕!'));
				});
			});
		});
	}).on('--help', function() {
		console.log('  举个栗子:');
		console.log('');
		console.log('bid deploy');
		console.log('');
		process.exit(1);
	});


program
	.command('dev')
	.alias('d')
	.description('进行开发')
	.option('-p, --port [type]', '监听端口', '3333')
	.action(function(cmd, options) {
		console.log(colors.green('开启开发者模式'));
		webServer.start({
			// port: program.port,
			port: program.args[0].port,
			queit: program.quiet
		});

		gitTools.setConfigVersion(); // 检测git分支，设置config.version

	}).on('--help', function() {
		console.log('  举个栗子:');
		console.log('');
		console.log('    bid dev,开启本地开发者模式');
		console.log('    bid dev -p|--port [端口号]   :   指定端口号');
		console.log('');
		process.exit(1);
	});

program
	.command('init')
	.alias('i')
	.description('初始化工程目录')
	.action(function(cmd, options) {
		console.log(colors.green('正在初始化工程目录ing...'));
		var dirname = path.join(process.cwd(), './');
		/*var dependencies = ' webpack gulp gulp-uglify del gulp-jshint gulp-inline-source gulp-htmlmin gulp-inline-css gulp-replace underscore gulp-util cli-color br-bid ';
		if (program.react) { // 初始化react工程
			dependencies += ' react react-dom redux react-redux redux-thunk';
		}*/
		utils.fileGenerator.projectGenerator({
			'dirname': dirname,
			'react': program.react
		}, function() { // 初始化常规工程
			console.log(colors.blue('正在安装工程构建所需要的依赖模块...'));
			var initTime = new Date().getTime();
			utils.fileGenerator.dependenciesGenerator({ // 复制依赖文件Node_modules
				'dirname': dirname
			}, function(error) {
				var nowTime = new Date().getTime();
				if (!error) {
					console.log(colors.green('依赖文件拷贝完成!'), colors.blue('共耗时:' + (nowTime - initTime) / 1000, 's'));
				} else {
					console.log(colors.red('拷贝依赖文件失败!'), colors.blue('共耗时:' + (nowTime - initTime) / 1000, 's'));
					console.log(colors.red(error));
				}
			});
		});
	}).on('--help', function() {
		console.log('  举个栗子:');
		console.log('');
		console.log('    bid init     ,   在当前路径下初始化[通用]工程目录');
		console.log('    bid init -r  ,   在当前路径下初始化[react]工程目录');
		console.log('');
		process.exit(1);
	});

program
	.command('update')
	.alias('u')
	.description('更新工程构建所需要的依赖模块')
	.action(function(cmd, options) {
		console.log(colors.blue('开始更新工程构建所需要的依赖模块...'));
		var initTime = new Date().getTime();
		var dirname = path.resolve('/usr/local/lib/node_modules');
		utils.fileGenerator.dependenciesGenerator({ // 复制依赖文件Node_modules
			'dirname': dirname
		}, function(error) {
			var nowTime = new Date().getTime();
			if (!error) {
				console.log(colors.green('依赖更新完成!'), colors.blue('共耗时:' + (nowTime - initTime) / 1000, 's'));
			} else {
				console.log(colors.red('拷贝依赖文件失败!'), colors.blue('共耗时:' + (nowTime - initTime) / 1000, 's'));
				console.log(colors.red(error));
			}
		});

	}).on('--help', function() {
		console.log('  举个栗子:');
		console.log('');
		console.log('    bid update  (-q|--quiet) ,   更新bid全局依赖模块(开启安静模式，默认"非安静模式")');
		console.log('    bid update -f [模块名],   强制重新安装bid指定依赖模块');
		console.log('');
		process.exit(1);
	});

program
	.command('iconfont')
	.description('iconfont ttf 2 base64')
	.option('-i, --input <file>', 'less或者css文件')
	.option('-o, --output <file>', '输出到此路径')
	.action(function(cmd, options) {
		console.log(colors.blue('现在开始 iconfont ttf 转换 base64 ...'));
		var parseUrl = function(text) {
			var urlRegex = /(https?:?)?(\/\/at.alicdn.com\/t\/font_.*\.ttf)/;
			if (urlRegex.exec(text)) {
				return 'http:' + RegExp.$2;
			}
			return false;
		};

		var parseLine = function(line) {
			return new Promise(function(resolve, reject) {
				var template = "    src: url(data:font/truetype;charset=utf-8;base64,<>) format('truetype');";
				var url = parseUrl(line);
				if (url) {
					urllib.request(url, function(err, data, res) {
						if (err) reject(err);
						var line = template.replace('<>', data.toString('base64'));
						resolve(line);
					})
				} else {
					resolve(line);
				}

			});
		};

		co(function*() {
			var input, output;
			if (program.args[0].input) {
				input = program.args[0].input;
			} else {
				process.exit(1);
			}
			var fileContent = fs.readFileSync(input).toString().split('\n');
			var arr = [];

			fileContent.forEach(function(line) {
				arr.push(parseLine(line));
			});
			var data = yield arr;

			// 有输出路径则写到对应文件，否则直接替换原文件
			if (program.args[0].output) {
				output = program.args[0].output;
				fs.writeFileSync(output, data.join('\n'));
				console.log(colors.green('替换完成！'));
			} else {
				fs.writeFileSync(input, data.join('\n'));
				console.log(colors.yellow('没有指定输出路径，源文件替换完成！'));
			}
		});

	}).on('--help', function() {
		process.exit(1);
	});

program
	.command('deploy1')
	.alias('p')
	.description('进行构建')
	.action(function(cmd, options) {
		var deployJSON = null; // build.json源
		var isPublish = false; // 是否构建后调用发布接口
		var configure = {}; // 读取自本地工程config.json配置
		/*var setConfigVersion = thunkify(function(callback) { // 检测当前git所处分支，并配置config.json
			gitTools.setConfigVersionThunk(false, function(err, config) {
				return callback(err, config);
			});
		});*/

		co(function*() {
			try {
				configure = yield setConfigVersion(false); // 检测git分支，设置config.version，并返回新的分支
			} catch (e) {
				console.log(colors.red('警告：当前不是git开发环境！'))
			}
			// configure = yield setConfigVersion(); // 检测git分支，设置config.version，并返回新的分支
			BUILDINFOS = utils.getBuildInfo(configure.version); // 使用新的git version重新更新buildInfo
			inquirer.prompt([{
				type: 'input',
				name: 'username',
				message: '请输入您的用户名:'
			}, {
				type: 'password',
				name: 'password',
				message: '请输入您的密码:'
			}, {
				type: 'list',
				name: 'env',
				message: '请选择发布环境:',
				choices: [{
						name: '本地生成部署配置',
						value: 'local'
					}, {
						name: '日常发布（相对路径构建，同时构建发布js、html）',
						value: 'daily'
					}, {
						name: '预发发布（相对路径构建，同时构建发布js、html）',
						value: 'pre'
					}, {
						name: '【本地构建】线上方式（在本地完成构建，但不发布）',
						value: 'production-build'
					}, {
						name: '线上发布（CDN路径构建，仅构建发布html）',
						value: 'production'
					}] //['本地生成部署配置', '日常环境', '预发环境', '线上环境']
			}, {
				type: 'checkbox',
				name: 'selectedEntry',
				message: '请选择需要进行构建的页面:',
				choices: BUILDINFOS.autoGetHtml.keys
			}]).then(function(answers) {
				deployJSON = answers;
				deployJSON.htmlEntry = [];
				deployJSON.jsEntry = {};
				deployJSON.appName = configure.appName; // 应用名
				deployJSON.remotes = configure.remotes; // Git远端地址
				deployJSON.version = configure.version; // Git分支版本
				deployJSON.publish = configure.publish; // 发布配置信息
				deployJSON.cdnhost = configure.cdnhost; // 静态资源cdn域名

				if (answers.selectedEntry.length) {
					answers.selectedEntry.forEach(function(se, index) { // 生成发布列表，构建列表
						for (var htmlKey in BUILDINFOS.autoGetHtml.jsEntry) {
							if (htmlKey.split(se).length > 1) {
								var tmpSrc = './' + htmlKey + '.html';
								if (configure.version) {
									tmpSrc = tmpSrc.replace(configure.version + '/', '');
								}
								if (BUILDINFOS.autoGetHtml.jsEntry[htmlKey]) {
									deployJSON.jsEntry[htmlKey] = BUILDINFOS.autoGetHtml.jsEntry[htmlKey];
								}
								deployJSON.htmlEntry.push(tmpSrc);
								break;
							}
						}
					});
					return deployJSON;
				} else {
					return false;
				}
			}).then(function(data) {
				console.log(data);
				if (data) {
					// var platformHost = 'http://platform.100credit.com';
					var platformHost = false;
					var deployDoAPI = false;
					co(function*() {
						var filename = path.join(utils.path.cwdPath, 'build.json');
						console.log(colors.green('gulp deploy --entry ' + filename + ' --env ' + data.env));
						var jsonData = JSON.stringify(data);
						try {
							fs.writeFileSync(filename, jsonData);
							console.log(colors.green('build.json创建成功'));
						} catch (err) {
							console.log(colors.red('build.json写入失败，请检查该文件'));
							console.log(colors.red(JSON.stringify(err)));
						}
						if (data.env === 'local') { // 默认只生成build.json
							isPublish = false;
							console.log(colors.green('生成发布配置成功，请查看工程目录下的build.json文件。'));
							try {
								yield execThunk('gulp deploy --entry ' + filename + ' --env daily'); // 在本地进行线上构建
								console.log(colors.green('本地日常构建完成!'));
							} catch (e) {
								console.log(colors.red('本地日常构建失败！'));
								console.log(e);
							}
						} else if (data.env === 'daily') {
							isPublish = true;
							try {
								platformHost = configure.publishAPI[data.env].host;
								deployDoAPI = platformHost + configure.publishAPI[data.env].deployDo;
							} catch (err) {
								return console.log(colors.red('config.js中的publishAPI.daily字段异常.发布结束'));
							}
						} else if (data.env === 'pre') {
							isPublish = true;
							try {
								platformHost = configure.publishAPI[data.env].host;
								deployDoAPI = platformHost + configure.publishAPI[data.env].deployDo;
							} catch (err) {
								return console.log(colors.red('config.js中的publishAPI.pre字段异常.发布结束'));
							}
						} else if (data.env === 'production') {
							isPublish = true;
							try {
								platformHost = configure.publishAPI[data.env].host;
								deployDoAPI = platformHost + configure.publishAPI[data.env].deployDo;
							} catch (err) {
								return console.log(colors.red('config.js中的publishAPI.production字段异常.发布结束'));
							}
						} else if (data.env === 'production-build') { // 使用线上构建方式，在本地完成构建，但不发布
							isPublish = false;
							try {
								yield execThunk('gulp deploy --entry ' + filename + ' --env production'); // 在本地进行线上构建
							} catch (e) {
								console.log(colors.red('本地线上构建失败！'));
								console.log(e);
							}
						}
						try {
							if (isPublish && platformHost && deployDoAPI) {
								var resp = yield handleApi({ // 调用发布接口
									url: deployDoAPI,
									form: data
								});
								var res = {};
								try {
									res = JSON.parse(resp[1]); // 获取res.body
								} catch (e) {
									console.log(colors.red('发布接口返回异常'));
									console.log(e);
								}
								if (res.code == 200) {
									console.log(colors.green('正在进行' + data.env + '发布:' + res.data.publishKey));
									console.log(colors.green('请在以下页面中查看发布进度: ' + platformHost + '/awp/logmonitor?f=' + data.appName + '/' + res.data.publishKey + '.log'));
								} else {
									console.log(colors.red('发布异常'));
									console.log(colors.red(JSON.stringify(res)));
								}
							}
						} catch (e) {
							console.log(colors.red('发布异常'));
							console.log(e);
						}
						console.log(colors.green('操作完成。'));
					});
				} else {
					console.log(colors.red('发布结束：您没有选择任何页面。'));
				}
			});
		});
	}).on('--help', function() {
		console.log('  举个栗子:');
		console.log('');
		console.log('bid deploy');
		console.log('');
		process.exit(1);
	});

program
	.command('deploy')
	.alias('p')
	.description('进行构建')
	.action(function(cmd, options) {
		var deployJSON = {
			htmlEntry: [],
			jsEntry: {}
		};
		var BUILDINFOS = false;
		var isPublish = false; // 是否构建后调用发布接口
		var configure = {}; // 读取自本地工程config.json配置
		var writeBuildJsonFile = function(filepath, data) {
			var jsonData = JSON.stringify(data);
			try {
				fs.writeFileSync(filepath, jsonData);
				console.log(colors.green('生成发布配置成功，请查看工程目录下的build.json文件。'));
			} catch (err) {
				console.log(colors.red('build.json写入失败，请检查该文件'));
				console.log(colors.red(JSON.stringify(err)));
			}
		};
		var getDeployServerInfos = function*() {
			var resp = yield handleApi({ // 调用发布接口
				url: `${getAPIHost('production')}/api/awp/getDeployServerInfo.do`,
				form: {}
			});
			var res = {};
			try {
				res = JSON.parse(resp[1]); // 获取res.body
			} catch (e) {
				console.log(colors.red('获取发布服务器信息返回解析异常'));
				console.log(e);
			}
			if (res.success) {
				console.log(colors.green('获取发布服务器信息成功'));
				return res.data;
			} else {
				console.log(colors.red('获取发布服务器信息返回异常'));
				console.log(colors.red(JSON.stringify(res)));
				return false;
			}
		};

		co(function*() {
			

			// try {
			// 	configure = yield setConfigVersion(deployJSON.tagBranch); // 检测git分支，设置config.version，并返回新的分支
			// } catch (e) {
			// 	console.log(colors.red('警告：当前不是git开发环境！'));
			// }
			// BUILDINFOS = utils.getBuildInfo(configure.version); // 使用新的git version重新更新buildInfo
			// deployJSON.htmlEntry = [];
			// deployJSON.jsEntry = {};
			// deployJSON.appName = configure.appName; // 应用名
			// deployJSON.remotes = configure.remotes; // Git远端地址
			// deployJSON.version = configure.version; // Git分支版本
			// deployJSON.websiteHost = configure.websiteHost; // 线上站点域名
			// deployJSON.cdnhost = configure.cdnhost; // 静态资源cdn域名

			var deployServerInfos = yield getDeployServerInfos(); // 获取发布环境及服务器信息

			var prompts = new Rx.Subject();
			var addServerPrompts = function(env) { // 检测是否获取了服务器信息，如果获取异常，且为日常、预发、线上发布，则退出
				if (!deployServerInfos) {
					return prompts.onError('获取发布服务器信息异常，云端发布失败');
				}
				if (env === 'daily') {
					prompts.onNext({
						type: 'list',
						name: 'publishGroup',
						message: '请选择日常环境类型:',
						choices: deployServerInfos.daily
					});
				} else if (env === 'pre') {
					prompts.onNext({
						type: 'list',
						name: 'publishGroup',
						message: '请选择预发服务器类型:',
						choices: deployServerInfos.pre
					});
				} else if (env === 'production') {
					prompts.onNext({
						type: 'list',
						name: 'publishGroup',
						message: '请选择线上服务器类型:',
						choices: deployServerInfos.production
					});
				}
			};
			var addUserInfoPrompts = function() { // 增加输入用户名、密码 inputer
				prompts.onNext({
					type: 'input',
					name: 'username',
					message: '请输入您的用户名:'
				});
				prompts.onNext({
					type: 'password',
					name: 'password',
					message: '请输入您的密码:'
				});
			};
			var getBuildInfoConfig = function*() {
				try {
					configure = yield setConfigVersion(deployJSON.tagBranch); // 检测git分支，设置config.version，并返回新的分支
					BUILDINFOS = utils.getBuildInfo(configure.version); // 使用新的git version重新更新buildInfo
					deployJSON.appName = configure.appName; // 应用名
					deployJSON.remotes = configure.remotes; // Git远端地址
					deployJSON.version = configure.version; // Git分支版本
					deployJSON.websiteHost = configure.websiteHost; // 线上站点域名
					deployJSON.cdnhost = configure.cdnhost; // 静态资源cdn域名
				} catch (e) {
					console.log(colors.red('警告：当前不是git开发环境！'));
					console.log(e);
				}
			};
			inquirer.prompt(prompts).ui.process.subscribe(
				function(data) { // onEachAnswer
					co(function*() {
						var answer = data.answer;
						if (data.name === 'env') {
							if (answer === 'daily' || answer === 'pre' || answer === 'production') {
								deployJSON.env = answer;
								addServerPrompts(answer);
								addUserInfoPrompts();
								yield getBuildInfoConfig();
								prompts.onNext({
									type: 'checkbox',
									name: 'selectedEntry',
									message: '请选择需要进行构建的页面:',
									choices: BUILDINFOS.autoGetHtml.keys
								});
							} else if (answer === 'localBuild') {
								prompts.onNext({
									type: 'list',
									name: 'envNext',
									message: '请选择本地构建方式:',
									choices: [{
										name: '[日常方式] 本地构建',
										value: 'local'
									}, {
										name: '[线上方式] 本地构建',
										value: 'production-build'
									}, {
										name: '[Tag方式] 本地构建',
										value: 'tag'
									}]
								});
							} else {
								prompts.onError(`非法环境：[ ${answer} ]`);
							}
						}

						if (data.name === 'envNext') {
							deployJSON.env = answer;
							if (answer === 'tag') {
								prompts.onNext({
									type: 'input',
									name: 'tagBranch',
									message: '请输入tag分支号（如：publish/1.0.0）:'
								});
							} else {
								yield getBuildInfoConfig();
								prompts.onNext({
									type: 'checkbox',
									name: 'selectedEntry',
									message: '请选择需要进行构建的页面:',
									choices: BUILDINFOS.autoGetHtml.keys
								});
							}
						}

						if (data.name === 'tagBranch') {
							deployJSON.tagBranch = answer;
							yield getBuildInfoConfig();
							prompts.onNext({
								type: 'checkbox',
								name: 'selectedEntry',
								message: '请选择需要进行构建的页面:',
								choices: BUILDINFOS.autoGetHtml.keys
							});
						}

						if (data.name === 'publishGroup') {
							deployJSON.publish = answer;
						}
						if (data.name === 'username') {
							if (answer) {
								deployJSON.username = answer;
							} else {
								prompts.onError('未输入用户名');
							}
						}
						if (data.name === 'password') {
							if (answer) {
								deployJSON.password = answer;
							} else {
								prompts.onError('未输入密码');
							}
						}
						if (data.name === 'selectedEntry') {
							if (answer.length > 0) {
								deployJSON.selectedEntry = answer;
								answer.forEach(function(se, index) { // 生成发布列表，构建列表
									for (var htmlKey in BUILDINFOS.autoGetHtml.jsEntry) {
										if (htmlKey.split(se).length > 1) {
											var tmpSrc = './' + htmlKey + '.html';
											if (configure.version) {
												tmpSrc = tmpSrc.replace(configure.version + '/', '');
											}
											if (BUILDINFOS.autoGetHtml.jsEntry[htmlKey]) {
												deployJSON.jsEntry[htmlKey] = BUILDINFOS.autoGetHtml.jsEntry[htmlKey];
											}
											deployJSON.htmlEntry.push(tmpSrc);
											break;
										}
									}
								});
								prompts.onCompleted(deployJSON);
							} else {
								prompts.onError('没有选择任何页面，操作结束。');
							}
						}
					});
				},
				function(data) { // onError
					console.log(colors.red(data));
				},
				function() { // onComplete
					console.log(colors.blue('输入完成，正在生成发布配置文件...'));
					var APIURL = false;
					var filename = path.join(utils.path.cwdPath, 'build.json');
					co(function*() {
						writeBuildJsonFile(filename, deployJSON);
						if (deployJSON.env === 'local' || deployJSON.env === 'production-build' || deployJSON.env === 'tag') { // 默认只生成build.json
							var buildName = (deployJSON.env === 'local') ? '日常' : ((deployJSON.env === 'production-build') ? '线上' : 'Tag');
							var buildType = (deployJSON.env === 'local') ? 'daily' : deployJSON.env;
							try {
								console.log(colors.blue(`开始进行本地[${buildName}]构建...`));
								console.log('本地构建，deployJSON配置文件如下：')
								console.log(deployJSON)
								console.log(colors.blue(`gulp deploy --entry ${filename} --env ${buildType}`));
								yield execThunk(`gulp deploy --entry ${filename} --env ${buildType}`); // 在本地进行线上构建
								console.log(colors.green(`本地[${buildName}方式]构建完成!`));
							} catch (e) {
								console.log(colors.red(`本地[${buildName}方式]构建失败！`));
								console.log(e);
							}
						} else if (deployJSON.env === 'daily' || deployJSON.env === 'pre' || deployJSON.env === 'production') {
							console.log(colors.magentaBright('您可以手动执行构建命令：gulp deploy --entry ' + filename + ' --env ' + deployJSON.env));
							APIURL = `${getAPIHost(deployJSON.env)}/api/awp/publish.do`;
						}
						if (APIURL) {
							try {
								// console.log('=====调用发布接口=====');
								// console.log(APIURL);
								// console.log(deployJSON);
								var platformHost = getAPIHost(deployJSON.env);
								var resp = yield handleApi({ // 调用发布接口
									url: APIURL,
									form: deployJSON
								});
								var res = {};
								try {
									res = JSON.parse(resp[1]); // 获取res.body
									// console.log(res);
								} catch (e) {
									console.log(colors.red('发布接口返回异常'));
									console.log(e);
								}
								if (res.success) {
									console.log(colors.green('正在进行' + deployJSON.env + '发布:' + res.data.publishKey));
									console.log(colors.green('请在以下页面中查看发布进度: ' + platformHost + '/awp/logmonitor?f=' + res.data.appName + '/' + res.data.publishKey + '.log'));
								} else {
									console.log(colors.red('发布异常'));
									console.log(colors.red(JSON.stringify(res)));
								}
							} catch (e) {
								console.log(colors.red('调用awp发布接口异常'));
								console.log(e);
							}
						}
					});

				}
			);
			prompts.onNext({
				type: 'list',
				name: 'env',
				message: '请选择发布环境:',
				choices: [{
					name: '日常发布（相对路径构建，同时构建发布js、html）',
					value: 'daily'
				}, {
					name: '预发发布（相对路径构建，同时构建发布js、html）',
					value: 'pre'
				}, {
					name: '线上发布（CDN路径构建，仅构建发布html）',
					value: 'production'
				}, {
					name: '本地构建（仅在本地进行构建，但不上传发布）',
					value: 'localBuild'
				}]
			});
		});
	}).on('--help', function() {
		console.log('  举个栗子:');
		console.log('');
		console.log('bid deploy');
		console.log('');
		process.exit(1);
	});


/*
 * bid tag
 * 描述：由git hock触发shell脚本，shell拉取指定tag之后，调用bid tag,
 * 参数说明：
 *	-b [分支号] : <require> 要构建的publish tag分支号。必填
 *	-l : <optional> 是否进行本地构建。可选
 *	-k [发布ID（publishKey）] : <optional> 发布ID。当在本地构建时，此项无用；当执行发布任务时，则为必填，由git hooks出发发布任务，由发布服务端调用脚本执行bid tag
 */
program
	.command('tag1')
	.alias('t')
	.option('-b, --branch [branch]', 'tag版本号')
	.option('-l, --local [local]', '是否进行本地构建', false)
	.option('-k, --publishkey [publishkey]', '发布ID（publishKey）', false)
	.description('执行git tag，通过sh，全量发布Javascript至cdn；并且使用线上cdn js资源构建及发布html到预发环境')
	.parse(process.argv)
	.action(function(cmd, options) {
		var deployJSON = {};
		var configure = {};
		var tagBranch = program.args[0].branch;
		var localBuild = program.args[0].local;
		var publishKey = program.args[0].publishkey;
		publishKey = (publishKey === true) ? false : publishKey;

		if (!tagBranch) { // 没有传递tag分支的名称，则返回错误
			console.log(colors.red('error'));
			console.log(colors.red('tag分支名称为空，失败'));
		} else {
			co(function*() {
				try {
					configure = yield setConfigVersion(tagBranch); // 检测git分支，设置config.version，并返回新的分支
				} catch (e) {
					console.log(colors.red('警告：当前不是git开发环境！'))
				}
				BUILDINFOS = utils.getBuildInfo(configure.version); // 使用新的git version重新更新buildInfo
				deployJSON.jsEntry = BUILDINFOS.autoGetEntry;
				// deployJSON.htmlEntry = []; // 数组为空时，gulp默认构建全部./src/p/**/*.html
				deployJSON.htmlEntry = BUILDINFOS.autoGetHtml.originList; // 取全部*.html，进行tag预发发布，并写入config.json中，用于生成及判断子任务upload状态
				deployJSON.version = configure.version; // Git分支版本
				deployJSON.appName = configure.appName; // 应用名
				deployJSON.remotes = configure.remotes; // Git远端地址
				deployJSON.publish = configure.publish; // 发布配置信息
				deployJSON.cdnhost = configure.cdnhost; // 静态资源cdn域名
				deployJSON.env = 'tag'; // 将环境设置为Tag

				var filename = path.join(utils.path.cwdPath, 'build.json');
				var jsonData = JSON.stringify(deployJSON);
				try {
					fs.writeFileSync(filename, jsonData);
					console.log(colors.green('build.json创建成功'));
				} catch (err) {
					console.log(colors.red('build.json写入失败，请检查该文件'));
					console.log(colors.red(err));
				}
				var commands = 'gulp deploy --entry ' + filename + ' --env tag';
				console.log(colors.green(commands));

				if (localBuild) { // 本地构建
					var initTime = new Date().getTime();
					exec(commands, {
						async: true,
						silent: program.quiet
					}, function(code, output) {
						var nowTime = new Date().getTime();
						console.log(colors.blue('耗时:' + (nowTime - initTime) / 1000, 's'));
						console.log(colors.green('javascripts资源构建完毕!'));
					});
				} else { // 上传配置，云端构建及发布
					if (publishKey) {
						deployJSON.publishKey = publishKey; // 发布ID
						try {
							var platformHost = configure.publishAPI.tag.host;
							var deployDoAPI = platformHost + configure.publishAPI.tag.deployDo;
							var resp = yield handleApi({ // 调用发布接口
								url: deployDoAPI,
								form: deployJSON
							});
							var res = {};
							try {
								res = JSON.parse(resp[1]); // 获取res.body
							} catch (e) {
								console.log(colors.red('发布接口返回异常'));
								console.log(e);
								console.log("error"); // 通知shell执行失败
							}
							if (res.code == 200) {
								console.log("success"); // 通知shell执行成功
							} else {
								console.log(colors.red('发布异常'));
								console.log(colors.red(JSON.stringify(res)));
								console.log("error"); // 通知shell执行失败
							}
						} catch (e) {
							console.log(colors.red('发布异常'));
							console.log(e);
							console.log("error"); // 通知shell执行失败
						}
					} else {
						console.log(colors.red('发布异常,没有publishKey'));
						console.log("error"); // 通知shell执行失败
					}
				}

			});
		}

	}).on('--help', function() {
		console.log('  举个栗子:');
		console.log('');
		console.log('bid tag');
		console.log('');
		process.exit(1);
	});

program
	.command('tag')
	.alias('t')
	.option('-b, --branch [branch]', 'tag版本号')
	.option('-l, --local [local]', '是否进行本地构建', false)
	.option('-k, --publishkey [publishkey]', '发布ID（publishKey）', false)
	.description('执行git tag，通过sh，全量发布Javascript至cdn；并且使用线上cdn js资源构建及发布html到预发环境')
	.parse(process.argv)
	.action(function(cmd, options) {
		var deployJSON = {};
		var configure = {};
		var tagBranch = program.args[0].branch;
		var localBuild = program.args[0].local;
		var publishKey = program.args[0].publishkey;
		publishKey = (publishKey === true) ? false : publishKey;

		if (!tagBranch) { // 没有传递tag分支的名称，则返回错误
			console.log(colors.red('error'));
			console.log(colors.red('tag分支名称为空，失败'));
		} else {
			co(function*() {
				try {
					configure = yield setConfigVersion(tagBranch); // 检测git分支，设置config.version，并返回新的分支
				} catch (e) {
					console.log(colors.red('警告：当前不是git开发环境！'))
				}
				BUILDINFOS = utils.getBuildInfo(configure.version); // 使用新的git version重新更新buildInfo
				deployJSON.jsEntry = BUILDINFOS.autoGetEntry;
				// deployJSON.htmlEntry = []; // 数组为空时，gulp默认构建全部./src/p/**/*.html
				deployJSON.htmlEntry = BUILDINFOS.autoGetHtml.originList; // 取全部*.html，进行tag预发发布，并写入config.json中，用于生成及判断子任务upload状态
				deployJSON.version = configure.version; // Git分支版本
				deployJSON.appName = configure.appName; // 应用名
				deployJSON.remotes = configure.remotes; // Git远端地址
				deployJSON.websiteHost = configure.websiteHost; // 线上站点域名
				deployJSON.cdnhost = configure.cdnhost; // 静态资源cdn域名
				deployJSON.env = 'tag'; // 将环境设置为Tag

				var filename = path.join(utils.path.cwdPath, 'build.json');
				var jsonData = JSON.stringify(deployJSON);
				try {
					fs.writeFileSync(filename, jsonData);
					console.log(colors.green('build.json创建成功'));
				} catch (err) {
					console.log(colors.red('build.json写入失败，请检查该文件'));
					console.log(colors.red(err));
				}
				var commands = 'gulp deploy --entry ' + filename + ' --env tag';
				console.log(colors.green(commands));

				if (localBuild) { // 本地构建
					console.log('本地tag构建')
					var initTime = new Date().getTime();
					exec(commands, {
						async: true,
						silent: program.quiet
					}, function(code, output) {
						var nowTime = new Date().getTime();
						console.log(colors.blue('耗时:' + (nowTime - initTime) / 1000, 's'));
						console.log(colors.green('javascripts资源构建完毕!'));
					});
				} else { // 上传配置，云端构建及发布
					if (publishKey) {
						console.log('远端tag构建');
						deployJSON.publishKey = publishKey; // 发布ID
						try {
							var deployDoAPI = `${getAPIHost('production')}/api/awp/publishTag.do`;
							console.log({
								url: deployDoAPI,
								form: deployJSON
							});
							var resp = yield handleApi({ // 调用发布接口
								url: deployDoAPI,
								form: deployJSON
							});
							var res = {};
							try {
								res = JSON.parse(resp[1]); // 获取res.body
							} catch (e) {
								console.log(colors.red('发布接口返回异常'));
								console.log(e);
								console.log("error"); // 通知shell执行失败
							}
							if (res.success) {
								console.log("success"); // 通知shell执行成功
							} else {
								console.log(colors.red('发布异常'));
								console.log(colors.red(JSON.stringify(res)));
								console.log("error"); // 通知shell执行失败
							}
						} catch (e) {
							console.log(colors.red('发布异常'));
							console.log(e);
							console.log("error"); // 通知shell执行失败
						}
					} else {
						console.log(colors.red('发布异常,没有publishKey'));
						console.log("error"); // 通知shell执行失败
					}
				}

			});
		}

	}).on('--help', function() {
		console.log('  举个栗子:');
		console.log('');
		console.log('bid tag');
		console.log('');
		process.exit(1);
	});


program.parse(process.argv);